@@ -124,7 +124,7 @@ class PushInfo(object):
         try:
             flags |= cls._flag_map[control_character]
         except KeyError:
-            raise ValueError("Control Character %r unknown as parsed from line %r" % (control_character, line))
+            raise ValueError("Control character %r unknown as parsed from line %r" % (control_character, line))
         # END handle control character
 
         # from_to handling
@@ -388,7 +388,7 @@ class Remote(LazyMixin, Iterable):
         if attr == "_config_reader":
             # NOTE: This is cached as __getattr__ is overridden to return remote config values implicitly, such as
             # in print(r.pushurl)
-            self._config_reader = SectionConstraint(self.repo.config_reader(), self._config_section_name())
+            self._config_reader = SectionConstraint(self.repo.config_reader("repository"), self._config_section_name())
         else:
             super(Remote, self)._set_cache_(attr)
 
@@ -455,7 +455,13 @@ class Remote(LazyMixin, Iterable):
             remote side, but are still available locally.
 
             The IterableList is prefixed, hence the 'origin' must be omitted. See
-            'refs' property for an example."""
+            'refs' property for an example.
+
+            To make things more complicated, it can be possble for the list to include
+            other kinds of references, for example, tag references, if these are stale
+            as well. This is a fix for the issue described here:
+            https://github.com/gitpython-developers/GitPython/issues/260
+            """
         out_refs = IterableList(RemoteReference._id_attribute_, "%s/" % self.name)
         for line in self.repo.git.remote("prune", "--dry-run", self).splitlines()[2:]:
             # expecting
@@ -463,8 +469,14 @@ class Remote(LazyMixin, Iterable):
             token = " * [would prune] "
             if not line.startswith(token):
                 raise ValueError("Could not parse git-remote prune result: %r" % line)
-            fqhn = "%s/%s" % (RemoteReference._common_path_default, line.replace(token, ""))
-            out_refs.append(RemoteReference(self.repo, fqhn))
+            ref_name = line.replace(token, "")
+            # sometimes, paths start with a full ref name, like refs/tags/foo, see #260
+            if ref_name.startswith(Reference._common_path_default + '/'):
+                out_refs.append(SymbolicReference.from_path(self.repo, ref_name))
+            else:
+                fqhn = "%s/%s" % (RemoteReference._common_path_default, ref_name)
+                out_refs.append(RemoteReference(self.repo, fqhn))
+            # end special case handlin
         # END for each line
         return out_refs
 
@@ -477,7 +489,9 @@ class Remote(LazyMixin, Iterable):
         :param kwargs: Additional arguments to be passed to the git-remote add command
         :return: New Remote instance
         :raise GitCommandError: in case an origin with that name already exists"""
-        repo.git.remote("add", name, url, **kwargs)
+        scmd = 'add'
+        kwargs['insert_kwargs_after'] = scmd
+        repo.git.remote(scmd, name, url, **kwargs)
         return cls(repo, name)
 
     # add is an alias
@@ -517,12 +531,13 @@ class Remote(LazyMixin, Iterable):
             Additional arguments passed to git-remote update
 
         :return: self """
-        self.repo.git.remote("update", self.name)
+        scmd = 'update'
+        kwargs['insert_kwargs_after'] = scmd
+        self.repo.git.remote(scmd, self.name, **kwargs)
         return self
 
     def _get_fetch_info_from_stderr(self, proc, progress):
         # skip first line as it is some remote info we are not interested in
-        # TODO: Use poll() to process stdout and stderr at same time
         output = IterableList('name')
 
         # lines which are no progress are fetch info lines
@@ -535,11 +550,10 @@ class Remote(LazyMixin, Iterable):
 
         progress_handler = progress.new_message_handler()
 
-        for line in proc.stderr:
+        for line in proc.stderr.readlines():
             line = line.decode(defenc)
-            line = line.rstrip()
             for pline in progress_handler(line):
-                if line.startswith('fatal:'):
+                if line.startswith('fatal:') or line.startswith('error:'):
                     raise GitCommandError(("Error when fetching: %s" % line,), 2)
                 # END handle special messages
                 for cmd in cmds:
@@ -550,8 +564,6 @@ class Remote(LazyMixin, Iterable):
                 # end for each comand code we know
             # end for each line progress didn't handle
         # end
-
-        # We are only interested in stderr here ...
         finalize_process(proc)
 
         # read head information
@@ -560,8 +572,9 @@ class Remote(LazyMixin, Iterable):
         fp.close()
 
         # NOTE: We assume to fetch at least enough progress lines to allow matching each fetch head line with it.
-        assert len(fetch_info_lines) >= len(fetch_head_info), "len(%s) <= len(%s)" % (fetch_head_info,
-                                                                                      fetch_info_lines)
+        l_fil = len(fetch_info_lines)
+        l_fhi = len(fetch_head_info)
+        assert l_fil >= l_fhi, "len(%s) <= len(%s)" % (l_fil, l_fhi)
 
         output.extend(FetchInfo._from_line(self.repo, err_line, fetch_line)
                       for err_line, fetch_line in zip(fetch_info_lines, fetch_head_info))
@@ -572,7 +585,6 @@ class Remote(LazyMixin, Iterable):
         # we hope stdout can hold all the data, it should ...
         # read the lines manually as it will use carriage returns between the messages
         # to override the previous one. This is why we read the bytes manually
-        # TODO: poll() on file descriptors to know what to read next, process streams concurrently
         progress_handler = progress.new_message_handler()
         output = IterableList('name')
 
@@ -585,9 +597,25 @@ class Remote(LazyMixin, Iterable):
             # END exception handling
         # END for each line
 
-        handle_process_output(proc, stdout_handler, progress_handler, finalize_process)
+        try:
+            handle_process_output(proc, stdout_handler, progress_handler, finalize_process)
+        except Exception:
+            if len(output) == 0:
+                raise
         return output
 
+    def _assert_refspec(self):
+        """Turns out we can't deal with remotes if the refspec is missing"""
+        config = self.config_reader
+        try:
+            if config.get_value('fetch', default=type) is type:
+                msg = "Remote '%s' has no refspec set.\n"
+                msg += "You can set it as follows:"
+                msg += " 'git config --add \"remote.%s.fetch +refs/heads/*:refs/heads/*\"'." % self.name
+                raise AssertionError(msg)
+        finally:
+            config.release()
+
     def fetch(self, refspec=None, progress=None, **kwargs):
         """Fetch the latest changes for this remote
 
@@ -615,14 +643,15 @@ class Remote(LazyMixin, Iterable):
         :note:
             As fetch does not provide progress information to non-ttys, we cannot make
             it available here unfortunately as in the 'push' method."""
+        self._assert_refspec()
         kwargs = add_progress(kwargs, self.repo.git, progress)
         if isinstance(refspec, list):
             args = refspec
         else:
             args = [refspec]
 
-        proc = self.repo.git.fetch(self, *args, with_extended_output=True, as_process=True, v=True, **kwargs)
-        proc.stdout.close()
+        proc = self.repo.git.fetch(self, *args, as_process=True, with_stdout=False, v=True,
+                                   **kwargs)
         res = self._get_fetch_info_from_stderr(proc, progress or RemoteProgress())
         if hasattr(self.repo.odb, 'update_cache'):
             self.repo.odb.update_cache()
@@ -636,9 +665,9 @@ class Remote(LazyMixin, Iterable):
         :param progress: see 'push' method
         :param kwargs: Additional arguments to be passed to git-pull
         :return: Please see 'fetch' method """
+        self._assert_refspec()
         kwargs = add_progress(kwargs, self.repo.git, progress)
-        proc = self.repo.git.pull(self, refspec, with_extended_output=True, as_process=True, v=True, **kwargs)
-        proc.stdout.close()
+        proc = self.repo.git.pull(self, refspec, with_stdout=False, as_process=True, v=True, **kwargs)
         res = self._get_fetch_info_from_stderr(proc, progress or RemoteProgress())
         if hasattr(self.repo.odb, 'update_cache'):
             self.repo.odb.update_cache()
